'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = createFormReducer;

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var _actionTypes = require('./actionTypes');

var initialState = {
  initial: {},
  data: {},
  touched: {},
  asyncValidating: false,
  asyncErrors: { valid: true },
  submitting: false
};

exports.initialState = initialState;
/**
 * Creates a state structure like:
 * {
 *   initial: {
 *     field1: 'value1',
 *     field2: 'value2'
 *   },
 *   data: {
 *     field1: 'value1',
 *     field2: 'value2'
 *   },
 *   touched: {
 *     field1: true,
 *     field2: false
 *   }
 * }
 *
 * @param name the name of the "state slice" where the data is stored
 * @param fields an array of field names, used when showing all values
 * @param config {
 *   touchOnBlur: [defaults to true],
 *   touchOnChange: [defaults to false]
 * }
 * @returns {Function} a form reducer
 */

function createFormReducer(name, fields) {
  var _ref = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var _ref$touchOnBlur = _ref.touchOnBlur;
  var touchOnBlur = _ref$touchOnBlur === undefined ? true : _ref$touchOnBlur;
  var _ref$touchOnChange = _ref.touchOnChange;
  var touchOnChange = _ref$touchOnChange === undefined ? false : _ref$touchOnChange;

  var reducer = function reducer(s) {
    var _extends2, _extends4;

    var action = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    var state = s || initialState;
    switch (action.type) {
      case _actionTypes.BLUR:
        var blurDiff = {
          data: _extends({}, state.data, (_extends2 = {}, _extends2[action.field] = action.value, _extends2))
        };
        if (touchOnBlur) {
          var _extends3;

          blurDiff.touched = _extends({}, state.touched, (_extends3 = {}, _extends3[action.field] = true, _extends3));
        }
        return _extends({}, state, blurDiff);
      case _actionTypes.CHANGE:
        var _state$asyncErrors = state.asyncErrors,
            oldError = _state$asyncErrors[action.field],
            valid = _state$asyncErrors.valid,
            otherErrors = _objectWithoutProperties(_state$asyncErrors, [action.field, 'valid']);

        var changeDiff = {
          data: _extends({}, state.data, (_extends4 = {}, _extends4[action.field] = action.value, _extends4)),
          asyncErrors: _extends({}, otherErrors, {
            valid: !Object.keys(otherErrors).length
          })
        };
        delete changeDiff.asyncErrors[action.field];

        if (touchOnChange) {
          var _extends5;

          changeDiff.touched = _extends({}, state.touched, (_extends5 = {}, _extends5[action.field] = true, _extends5));
        }
        return _extends({}, state, changeDiff);
      case _actionTypes.INITIALIZE:
        return {
          initial: action.data,
          data: action.data,
          asyncValidating: false,
          asyncErrors: {},
          touched: {}
        };
      case _actionTypes.RESET:
        return {
          initial: state.initial,
          data: state.initial,
          touched: {},
          asyncValidating: false,
          asyncErrors: {}
        };
      case _actionTypes.START_ASYNC_VALIDATION:
        return _extends({}, state, {
          asyncValidating: true
        });
      case _actionTypes.START_SUBMIT:
        return _extends({}, state, {
          submitting: true
        });
      case _actionTypes.STOP_ASYNC_VALIDATION:
        return _extends({}, state, {
          asyncValidating: false,
          asyncErrors: action.errors
        });
      case _actionTypes.STOP_SUBMIT:
        return _extends({}, state, {
          submitting: false
        });
      case _actionTypes.TOUCH:
        var touchDiff = {};
        action.fields.forEach(function (field) {
          if (typeof field !== 'string') {
            throw new Error('fields passed to touch() must be strings');
          }
          touchDiff[field] = true;
        });
        return _extends({}, state, {
          touched: _extends({}, state.touched, touchDiff)
        });
      case _actionTypes.TOUCH_ALL:
        var touchAllDiff = {};
        fields.forEach(function (field) {
          return touchAllDiff[field] = true;
        }); // mark all as touched
        return _extends({}, state, {
          touched: touchAllDiff
        });
      case _actionTypes.UNTOUCH:
        var untouchDiff = {};
        action.fields.forEach(function (field) {
          if (typeof field !== 'string') {
            throw new Error('fields passed to untouch() must be strings');
          }
          untouchDiff[field] = false;
        });
        return _extends({}, state, {
          touched: _extends({}, state.touched, untouchDiff)
        });
      case _actionTypes.UNTOUCH_ALL:
        return _extends({}, state, {
          touched: {}
        });
      default:
        return state;
    }
  };
  return function () {
    var state = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];
    var action = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    if (action.form !== name) {
      return state;
    }
    if (action.key) {
      var _extends6;

      return _extends({}, state, (_extends6 = {}, _extends6[action.key] = reducer(state ? state[action.key] : undefined, action), _extends6));
    }
    return reducer(state, action);
  };
}